#!/bin/bash
#

# buildPython.sh <product_dir> [tar] [source]

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    PRODUCTS=${product_dir}:${PRODUCTS}
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
    if [ -z ${SSIBUILDSHIMS_DIR} ]
    then
       echo "ERROR: failed to setup ssibuildshims ${ssibuildshims_version}"
       exit 1
    fi
}

get_platform_name()
{
    OS1=`uname`
    flvr=`ups flavor`
    if [ ${OS1} = "Darwin" ]
    then
	flvr=`ups flavor -2`
    fi
    OS=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform`
    plat=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osplat`

    if [ -z ${flvr} ]
    then
       echo "ERROR: could not determine flavor for this platform"
       echo "       something is very wrong"
       exit 1
    fi

    pkgdir=${product_dir}/${package}/${pkgver}/${flvr}
    patchdir=${product_dir}/${package}/${pkgver}/patch
    scriptsdir=${product_dir}/${package}/${pkgver}/scripts
    blddir=${product_dir}/${package}/${pkgver}/build/${flvr}
    tardir=${product_dir}/${package}/${pkgver}/tar
    sourcedir=${product_dir}/${package}/${pkgver}/${flvr}/src
}

die()
{
  echo "$1"; exit 2
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
maketar=${2}
sourcetar=${3}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------
set -x
package=python
origpkgver=v2_7_12
pkgver=${origpkgver}
ssibuildshims_version=v0_13_01
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pythonMmVer=`echo ${pkgdotver} | sed -e 's/^\([0-9]\{1,\}\.[0-9]\{1,\}\).*$/\1/'`
pkgtarname=Python-${pkgdotver}
set  +x
get_this_dir

get_ssibuildshims

if [ "${sourcetar}" = "source" ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_source_code_tarball ${product_dir} ${package} ${pkgver}
fi

get_platform_name

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
    ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
    exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# setup this package
setup -B ${package} ${pkgver} -z ${fakedb}:${product_dir}:${PRODUCTS} || die "ERROR: fake setup failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

if [ -z ${PYTHON_ROOT} ]
then
   echo "ERROR: PYTHON_ROOT is not defined"
   exit 1
else
   echo "PYTHON_ROOT: ${PYTHON_ROOT}"
fi

if [ -z ${SQLITE_INC} ]
then
   echo "ERROR: failed to setup sqlite"
   exit 1
else
   echo "SQLITE_INC: ${SQLITE_INC}"
fi

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tgz

cd ${sourcedir}/${pkgtarname}

declare -a patches=(${patchdir}/${pkgtarname}.patch)

# Note: sysconfigdata.patch is used by fixit.sh, whose invocation is
# inserted into the procedure by the main patch above. Do *not* attempt
# to apply it separately.

if [ ${OS1} = "Darwin" ]
then
:
  patches[${#patches[@]}]=${patchdir}/Darwin.setup.py.patch
  patches[${#patches[@]}]=${patchdir}/Darwin.Modules.Setup.patch
fi

if [ ${OS1} = "Linux" ]; then
  if [ "$(lsb_release -i -s)" = "SUSE LINUX" ]
  then
    patches[${#patches[@]}]=${patchdir}/SUSE.Modules.panel.patch
  fi
fi

for patch in "${patches[@]}"; do
  patch ${DEBUG_PATCHES:+--verbose} -b -p0 < "${patch}" || die "ERROR: application of patch ${patch} failed."
done

# We need to specify the installed location of SQLite so that the build finds ours, not a system install.
if [  -d /usr/local/opt/openssl ]
then
  ./configure LDFLAGS="-L${SQLITE_LIB} -L/usr/local/opt/openssl/lib" \
              CPPFLAGS="-I${SQLITE_INC} -I/usr/local/opt/openssl/include" \
              --prefix=${pkgdir} --enable-shared || die "ERRPR: configure failed."
else
  ./configure LDFLAGS=-L${SQLITE_LIB} CPPFLAGS=-I${SQLITE_INC} \
              --prefix=${pkgdir} --enable-shared || die "ERRPR: configure failed."
fi
make -j ${ncore} || die "ERROR: make failed."
make install || die "ERROR: make install failed."

# replace idle, pydoc, and smtpd.py with the scripts which use #!/usr/bin/env python
if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi
cd ${pkgdir}/bin
cp -p ${sourcedir}/${pkgtarname}/Tools/scripts/2to3 .
cp -p ${sourcedir}/${pkgtarname}/Tools/scripts/pydoc .
cp -p ${sourcedir}/${pkgtarname}/Tools/scripts/idle .
cp -p ${sourcedir}/${pkgtarname}/Lib/smtpd.py .
mv python${pythonMmVer}-config python${pythonMmVer}-config.bak
cat python${pythonMmVer}-config.bak | sed 's%^#!.*python'"${pythonMmVer//./\\.}"'$%#!/usr/bin/env python%' > python${pythonMmVer}-config
chmod +x python${pythonMmVer}-config

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} NO_QUAL

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver}  -z ${product_dir}
echo "${package} is installed at ${PYTHONHOME}"
echo ""

# fix paths in ${PYTHONHOME}/lib/python${pythonMmVer}/config/Makefile

configdir=${PYTHONHOME}/lib/python${pythonMmVer}/config

fixscript=`mktemp "\${TMPDIR:-/tmp}/python-fix.XXXXXXXXXX"`
trap "rm $fixscript 2>/dev/null" EXIT

echo "#!/bin/bash" > ${fixscript}
echo "" >>  ${fixscript}
echo "mv ${configdir}/Makefile ${configdir}/Makefile.bak" >> ${fixscript}
echo "cat ${configdir}/Makefile.bak \\" >>  ${fixscript}
echo "      | sed 's%${PYTHON_INCLUDE}%\$(PYTHON_INCLUDE)%' \\" >>  ${fixscript}
echo "      | sed 's%${PYTHON_LIB}%\$(PYTHON_LIB)%' \\" >>  ${fixscript}
echo "      | sed 's%${PYTHONHOME}%\$(PYTHONHOME)%' \\" >>  ${fixscript}
echo "      | sed 's%LD_LIBRARY_PATH=${PYTHON_DIR}/build/${flvr}%LD_LIBRARY_PATH=\$(PYTHON_LIB)%' \\" >>  ${fixscript}
echo "      | sed 's%${PYTHON_DIR}/source%\$(PYTHON_DIR)/source%' \\" >>  ${fixscript}
echo "      | sed 's%${PYTHON_DIR}%\$(PYTHON_DIR)%' \\" >>  ${fixscript}
echo "      | sed 's%${SQLITE_INC}%\$(SQLITE_INC)%' \\" >>  ${fixscript}
echo "      | sed 's%${SQLITE_LIB}%\$(SQLITE_LIB)%' \\" >>  ${fixscript}
echo "      | sed 's%${SQLITE_FQ_DIR}%\$(SQLITE_FQ_DIR)%' \\" >>  ${fixscript}
echo "      > ${configdir}/Makefile" >> ${fixscript}
echo "" >>  ${fixscript}

chmod +x ${fixscript}
${fixscript}
# cleanup
rm -f ${fixscript}

# Test that the library has built all the necessary components
echo "The Python we are running is at:" $(type python)
echo "Preparing to test loading of Python Standard Library modules"
python ${scriptsdir}/test-standard-modules.py || \
 die "ERROR: unable to find all expected Python Standard Library modules."
echo "Succeded in test loading of Python Standard Library modules"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver}  NO_QUAL
fi

exit 0
